locals {
  lambda_loc = "${path.module}/lambda"
}

resource "aws_lambda_function" "crud" {
  for_each                       = local.lambdas
  filename                       = data.archive_file.lambda_file.output_path
  function_name                  = "${local.prefix}-${each.key}-crud-dynamodb"
  role                           = aws_iam_role.iam_for_lambda.arn
  handler                        = "${each.key}.handler"
  timeout                        = 10
  source_code_hash               = filebase64sha256(data.archive_file.lambda_file.output_path)
  runtime                        = "nodejs12.x"
  reserved_concurrent_executions = 2
  layers = [
    "arn:aws:lambda:eu-west-1:580247275435:layer:LambdaInsightsExtension:14"
  ]

  tracing_config {
    mode = "Active"
  }

  environment {
    variables = {
      TABLE_NAME = aws_dynamodb_table.main.name
      TABLE_KEY  = aws_dynamodb_table.main.hash_key
    }
  }

  tags = local.common_tags
}

resource "aws_lambda_function" "index" {
  filename                       = data.archive_file.lambda_index_file.output_path
  function_name                  = "${local.prefix}-index"
  role                           = aws_iam_role.iam_for_lambda.arn
  handler                        = "index.handler"
  timeout                        = 10
  source_code_hash               = filebase64sha256(data.archive_file.lambda_index_file.output_path)
  runtime                        = "nodejs12.x"
  reserved_concurrent_executions = 2
  layers = [
    "arn:aws:lambda:eu-west-1:580247275435:layer:LambdaInsightsExtension:14"
  ]

  tags = local.common_tags
}

resource "aws_iam_role" "iam_for_lambda" {
  name               = "${local.prefix}-lambda"
  assume_role_policy = file("./templates/lambda/assume-role-policy.json")
  tags               = local.common_tags
}

# See also the following AWS managed policy: AWSLambdaBasicExecutionRole
resource "aws_iam_policy" "lambda_logging" {
  name        = "${local.prefix}-lambda_logging"
  path        = "/"
  description = "IAM policy for logging from a lambda"
  policy      = file("./templates/lambda/lambda-policy.json")
}

resource "aws_iam_role_policy_attachment" "lambda_logs" {
  role       = aws_iam_role.iam_for_lambda.name
  policy_arn = aws_iam_policy.lambda_logging.arn
}

resource "aws_iam_role_policy_attachment" "lambda_xray" {
  role       = aws_iam_role.iam_for_lambda.name
  policy_arn = "arn:aws:iam::aws:policy/AWSXRayDaemonWriteAccess"
}

resource "aws_iam_role_policy_attachment" "lambda_insights" {
  role       = aws_iam_role.iam_for_lambda.name
  policy_arn = "arn:aws:iam::aws:policy/CloudWatchLambdaInsightsExecutionRolePolicy"
}

data "archive_file" "lambda_file" {
  type        = "zip"
  output_path = "${local.lambda_loc}/zip/lambda.zip"
  source_dir  = "${local.lambda_loc}/src"
}

data "archive_file" "lambda_index_file" {
  type        = "zip"
  output_path = "${local.lambda_loc}/zip/index.zip"
  source_file = "${local.lambda_loc}/src/index.js"
}
